#####################################################################################################################
#Copyright IBM
#
# This PERL script was created by Shirley Dango of Rational Software.
# It was created on November 18, 2002 for the RATLC production database as part of the CC/CQ/UCM Integration project.
# The purpose is to create the Build record in the RATLC production database.
# The user must provide the user login id, password, database name (RATLC), database set name,
# pvob, build type, product, version and new baseline to create. 
######################################################################################################################
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003 
# Updated by Katie Carpenter, October & November 2007
# Updated by Stuart Poulin Nov 2007
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	build			The name of the build
#	buildstatus		The type of build 
#	projectid		The CQ id of the project this build is related to - for example ALM0000045
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
# thl3   RATLC00475598       10/19/2005      THL               SQUID Scripts for baseline and build create fail when 
#                                                              login is ldap enabled
# thl4   RATLC00475646       10/19/2005      THL               Need to check for \ on pvob parm 
# thl5   RATLC00475658       10/23/2005      THL               SQUID Script create_build_record allows multiple build 
#                                                              records to be created with the same build name 
#                                                              (Build field value)
#			     10/11/2007	     katherns	       Modify references of 'Build' to 'BTBuild'
#
#        RATLC00396109       10/26/2007      katherns          build cannot be created against an obsolete baseline
#
#        RATLC00396140       10/26/2007      katherns          build record does not need to contain activities
#
#        RATLC00396143       10/26/2007      katherns          change references of 'relCWActivities' to 'Activities'
#
#        RATLC01214353       10/26/2007      katherns          add Build_System_ID as a parameter (build)
#
#        RATLC01216069       11/5/2007       katherns          add ALMBuildType as a parameter
#
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#        RATLC               3/31/2008       spoulin           add -url option
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
my $InternalVersion = 0.6;
use strict;
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
my ($entity);
my $id;
my ($failure);
my ($baselineentity);
my $result;
my $progname = basename $0;
my $status ;
my $buildstatus ; #thl2
my $userLogin ; #thl2
my ($resultset,$resultsetstatus,$buildId);
my $recordOwner;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    print "ID $id\n";
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
	print "CQ API gpărh R[h쐬郆[eBeB
    gp@:
       $progname -test
         -help
         -user <[U[>
         -pw <pX[h>
         -dbname <f[^x[X>
         -dbset <f[^x[X Zbg>
         -pvob <pvob>
         -baseline <x[XC>
         -build <rh>
         -buildstatus <rh Xe[^X>
         -buildtype <rh̃^Cv>
         -projectid <vWFNg ID>
	 -logfile <O t@C>
	 -url <url_to_build>
"		
}

##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s
    pw=s
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    build=s
    buildstatus=s
    buildtype=s
    projectid=s
    owner=s
    logfile=s
    test
    help
    url=s
   
);


my %opts = (
    user => $ENV{ALMUSER},
    pw => $ENV{ALMPW},
    dbname => $ENV{ALMDBNAME},
    dbset => $ENV{ALMDBSET},
    pvob => $ENV{ALMPVOB},
    baseline => undef,
    build =>undef,
    buildstatus => undef,
    buildtype => undef,
    projectid => undef,
    owner => undef,
    logfile => undef,
    url=>undef,
);


my @required_opts = qw ( user pw dbname dbset pvob baseline buildstatus buildtype projectid build );

####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
 	$logger->log_print_message("R}hsւ̈p@:");	
      	my $retval = &Usage;
      	return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "O t@C fBNg $logdir 쐬ł܂ł");
            		$logger->log_print_error( __LINE__ . "XNvg̎śAȂŌp܂B");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "O t@C $opts{logfile} J܂łB");
        	$logger->log_print_error( __LINE__ . "XNvg̎śAȂŌp܂B");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("MO $log_time ɊJn܂B"); 
}	 

####
# Display the help usage
if ($opts{help}) {
	print "-user		ClearQuest [U[̃OC IDB\n";
	print "-pw			 ClearQuest [U[̃pX[hB\n";
	print "-dbname		XL[} |Wg̃f[^x[Xւ̐ڑB\n";
	print "-dbset		[U[ f[^x[XڑB\n";
	print "-pvob		vWFNg VOB ̖OB\n";
	print "-baseline	x[XC̖OB\n";
	print "-build		rh̖OB\n";
	print "-buildstatus	rh̃Xe[^XB\n";
        print "-buildtype       rh̃^CvB\n";
	print "-projectid	̃rh֘AĂvWFNg IDB\n";
	print '-logfile		́AO t@C̃pX`IvV tOłBftHg̓R\[Ɉ܂B';
	print "\n-url		́Arh URL 󂯓n߂ɐݒ肳IvV tOłB\n";
	print "-test		́AeXg t@C͂ɐݒłIvV tOłB\n";
	print "-help            ́AwṽIvV tOłB\n";
    return 1;
}

# Display test inputs to log file 03/26/03
if ($opts{test}){
	$logger->log_print_message("`ꂽ[U[:");
	$logger->log_print_message("`ꂽ[U[ OC $opts{user} łB");
	$logger->log_print_message("`ꂽ dbset  $opts{dbset} łB");
	$logger->log_print_message("`ꂽf[^x[X $opts{dbname} łB");
	$logger->log_print_message("`ꂽx[XC $opts{baseline} łB");
	$logger->log_print_message("`ꂽvWFNg VOB  $opts{pvob} łB");
	$logger->log_print_message("`ꂽrhԂ $opts{buildstatus} łB");
	$logger->log_print_message(" `ꂽrh ^Cv $opts{buildtype} łB");
	$logger->log_print_message("`ꂽvWFNg ID  $opts{projectid} łB");
	if ($opts{logfile}){ $logger->log_print_message("`ꂽO t@C $opts{logfile} łB");}
	return 1;
}

####
####
# End program if any of the required parameters are not defined
{my  $message = "";
	for my $req (@required_opts )  {
		$message .= "KvȃIvV -$req  ̌\n" if ! defined $opts{$req};
	}
	if ($message ) {
		$logger->log_print_message("KvȃR}hsIvVĂ܂AR}hsւ̈p@:");
		$logger->log_print_message("$message");
		print STDERR "$message\n";
		my $retval = &Usage();
		return 0;
	}
}
;


my ($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});
###
# Start a CQ session
my $session = CQPerlExt::CQSession_Build();

# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database
eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error( __LINE__ . "$opts{dbname} ւ̃OCɎs܂: $@"); 
return 0;
}
$userLogin = $session->GetUserLoginName() ; #thl3
$logger->log_print_message("$opts{dbname} ւ̃OC $userLogin ܂B"); #thl3


# See if the build record already exists and return if it does            thl5
my $queryDef = $session->BuildQuery("BTBuild");                            #thl5   
$failure = $queryDef->BuildField("ID");                                  #thl5
my $operator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND); #thl5 
$operator->BuildFilter("OldID", $CQPerlExt::CQ_COMP_OP_EQ, ["$opts{build}"]); #thl5
eval {$resultset = $session->BuildResultSet($queryDef)};                  #thl5
if ($@) {                                                                 #thl5
   $logger->log_print_error(__LINE__ . "NG[̃rhɎs܂: $@");   #thl5
   return 0;                                                              #thl5
}                                                                         #thl5
else {                                                                    #thl5
   $resultset->Execute();                                                 #thl5
   $resultsetstatus = $resultset->MoveNext;                               #thl5
   if ($resultsetstatus == $CQPerlExt::CQ_SUCCESS) {                      #thl5
      $buildId = $resultset->GetColumnValue(1);
      $logger->log_print_error(__LINE__ . "rh R[h $opts{build}  $buildId ƂĂłɑ݂܂");#thl5
      return 0;                                                           #thl5
   }                                                                      #thl5
}                                                                         #thl5

# Find the Baseline record in the ClearQuest database
my $baselineentity = undef;
for (1) {
	eval qq{\$baselineentity = \$session->GetEntity("ALMBaseline", \$noslashvob.' '.\$opts{baseline})}; 
	if ($baselineentity) {
		$opts{pvob} = $noslashvob;
		last;
	}
	eval qq{\$baselineentity = \$session->GetEntity("ALMBaseline", \$slashvob.' '.\$opts{baseline})}; 
	if ($baselineentity) {
		$opts{pvob} = $slashvob;
		last;
	}
	$baselineentity = $session->GetEntity("ALMBaseline", $opts{pvob}.' '.$opts{baseline}); 
}

$logger->log_print_message(" $opts{baseline} ̃x[XC GeBeBvWFNg VOB Ō܂: $opts{pvob}B");

# Determine if the Baseline record was found in ClearQuest database
if ($baselineentity){

#Do not create build if baseline is obsolete
my $obsoletevalue = $baselineentity->GetFieldValue("p~")->GetValue();
if($obsoletevalue eq 'Yes'){
$logger->log_print_message(" x[XC R[h͔p~܂Brh R[h͍쐬܂B");
$logger->log_print_error(" x[XC R[h͔p~܂Brh R[h͍쐬܂B");
return 0;
}

	$logger->log_print_message(" VKrh R[h쐬Ă܂B");
	
	## Build the Build record in ClearQuest database
	my $entity;
	eval {$entity = $session->BuildEntity("BTBuild")};
	if ($@) {
		$logger->log_print_error( __LINE__ . " rh R[hJ܂:"); 
		$logger->log_print_error( __LINE__ . " $@");
        return 0;
	}


	my $baselinename = $opts{pvob} . ' ' . $opts{baseline};
	
	## Set field values for pvob, baseline, buildstatus,project, and version
	
	$entity->SetFieldValue("ALMProject", $opts{projectid});
	$entity->SetFieldValue("Build_System_ID", $opts{build});	
	$entity->SetFieldValue("ALMBuildStatus",$opts{buildstatus});
        $entity->SetFieldValue("ALMBuildType", $opts{buildtype});
	$entity->SetFieldValue("ALMOwner", $userLogin); 
        $entity->SetFieldValue("ALMBaseline", $baselinename); 
	$entity->SetFieldValue("Build_System_URL", $opts{url});
			
	## New Build ID
	$id = $entity->GetFieldValue("ID")->GetValue();
	$logger->log_print_message("ID=$id");
	$logger->log_print_message(" VKrh R[h؂Ă܂B");
	$status = "" ; #thl2
	eval {$status = $entity->Validate()};
	if (($@) || ($status)) {           #thl2
		$entity->Revert();
		$logger->log_print_error(__LINE__ . "ClearQuest f[^x[XŃrh R[h؂ł܂łB");
		$logger->log_print_error(__LINE__ . "  eval=$@");
                $logger->log_print_error(__LINE__ . "߂Xe[^X $status łB"); #thl2
		$logger->log_print_error(__LINE__ . "R[hrh邽߂̂̏ł̓̓p[^͎̂Ƃł:");
		$logger->log_print_error("\tALMProject:", $opts{projectid});
		$logger->log_print_error("\tBuild_System_ID:", $opts{build});	
		$logger->log_print_error("\tALMBuildStatus:",$opts{buildstatus});
		$logger->log_print_error("\tALMBuildType:", $opts{buildtype});
		$logger->log_print_error("\tALMOwner:", $userLogin); 
		$logger->log_print_error("\tALMBaseline:", $baselinename); 
		return 0;
	}
        $status = "" ; #thl2
	$logger->log_print_message(" VKrh R[hR~bgĂ܂B");
	eval {$status = $entity->Commit()}; #thl2
	if (($@) || ($status)) {            #thl2
		$entity->Revert();
		$logger->log_print_error( __LINE__ . "ClearQuest f[^x[XŃrh R[hR~bgł܂łB");
		$logger->log_print_error( __LINE__ . " $@");
                $logger->log_print_error(__LINE__ . "߂Xe[^X $status łB"); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
        # Write to log file
	$logger->log_print_message(" VKrh R[h쐬܂B");
	$entity = undef;
	CQPerlExt::CQSession_Unbuild($session);
	} 
else {
	# Write to log file 
	$logger->log_print_message(" x[XC R[h܂BVKx[XC R[h쐬ĂB");
	$logger->log_print_message(" rh R[h͍쐬܂łB");	
	CQPerlExt::CQSession_Unbuild($session);
	return 0;
} # end of creating Build and updating Baseline record in CQ
# Write to log file

$logger->log_print_success("create_build_record XNvg܂B");
$logger->log_print_message(" create_build_record.pl XNvg܂");
CQPerlExt::CQSession_Unbuild($session);

return 1;
}
#### SUBROUTINES FOR PROGRAM
